/**
 * Copyright (c) 2009-2012 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.zauberlabs.commons.mom;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.List;

import net.sf.staccatocommons.check.Ensure;
import net.sf.staccatocommons.restrictions.check.NonNull;
import net.sf.staccatocommons.restrictions.check.NotEmpty;

/**
 * @author flbulgarelli
 */
public final class StructureType {

    /***/
    private StructureType() {
    }

    /**
     * Answers the structure type of a property, given an instance and the
     * property name, if it exists. Throws an exception otherwise
     *
     * The structure type is not the same that the property type. According to
     * SLM format, it follows these rules:
     * <ul>
     * <li>If the given property is a simple or composite type in the target
     * instance, its property type is returned</li>
     * <li>Otherwise, if it is a composite type, the component type is returned</li>
     * </ul>
     *
     * @param target
     * @param propertyName
     * @return the property type
     */
    @SuppressWarnings("rawtypes")
    public static Class<?> getStructureType(@NonNull final Object destination,
            @NotEmpty final String propertyName) {
        Type targetType = getType(destination, propertyName);

        if (targetType instanceof Class) {
            return (Class) targetType;
        }

        if (targetType instanceof ParameterizedType) {
            ParameterizedType parameterizedType = (ParameterizedType) targetType;
            if (parameterizedType.getRawType() == List.class) {
                return (Class) parameterizedType.getActualTypeArguments()[0];
            }
        }
        return Ensure.that().fail("Unsupported type %s for propertyName %s in object %s", targetType,
                (Object) propertyName, destination);
    }

    /** Gets {@link Type} of a given property */
    private static Type getType(final Object bean, final String propertyName) {
        return NaiveProperties.getter(bean.getClass(), propertyName).getGenericReturnType();
    }


}
