/**
 * Copyright (c) 2009-2012 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.zauberlabs.commons.mom.internal;

import java.util.LinkedList;
import java.util.List;

import net.sf.staccatocommons.check.Ensure;
import net.sf.staccatocommons.lang.Nulls;
import net.sf.staccatocommons.restrictions.check.NonNull;
import net.sf.staccatocommons.restrictions.check.NotEmpty;
import net.sf.staccatocommons.restrictions.processing.EnforceRestrictions;

import com.zauberlabs.commons.mom.MapObjectMapper;
import com.zauberlabs.commons.mom.MapObjectMapperBuilder;
import com.zauberlabs.commons.mom.MapObjectMapperInterceptor;
import com.zauberlabs.commons.mom.PropertyModel;
import com.zauberlabs.commons.mom.converter.CompositeTypeConverter;
import com.zauberlabs.commons.mom.converter.IdentityTypeConverter;
import com.zauberlabs.commons.mom.converter.TypeConverter;
import com.zauberlabs.commons.mom.style.NamingStyle;
import com.zauberlabs.commons.mom.style.SetterStyle;
import com.zauberlabs.commons.mom.style.impl.BeanNamingStyle;
import com.zauberlabs.commons.mom.style.impl.BeanStyle;

/**
 *
 * DefaultMapObjectMapperBuilder
 * @author flbulgarelli
 */
@EnforceRestrictions
public class DefaultMapObjectMapperBuilder implements MapObjectMapperBuilder {

    private final String packageName;
    private SetterStyle setterStyle;
    private NamingStyle namingStyle;
    private PropertyModel propertyModel;
    private List<TypeConverter> converters = new LinkedList<TypeConverter>();
    private MapObjectMapperInterceptor interceptor = null;

    /**
     * Creates a new {@link DefaultMapObjectMapperBuilder}
     */
    public DefaultMapObjectMapperBuilder(@NotEmpty final String packageName) {
        this.packageName = packageName;
        withConverter(IdentityTypeConverter.converter());
    }

    @Override
    public final MapObjectMapperBuilder withSetterStyle(@NonNull final SetterStyle style) {
        this.setterStyle = style;
        return this;
    }

    @Override
    public final MapObjectMapperBuilder withConverter(@NonNull final TypeConverter converter) {
        this.converters.add(converter);
        return this;
    }

    @Override
    public final MapObjectMapperBuilder withInterceptor(@NonNull final MapObjectMapperInterceptor inter) {
        this.interceptor = inter;
        return this;
    }

    @Override
    public final MapObjectMapperBuilder withNamingStyle(@NonNull final NamingStyle style) {
        this.namingStyle = style;
        return this;
    }

    @Override
    public final MapObjectMapperBuilder withPropertyModel(@NonNull final PropertyModel model) {
        this.propertyModel = model;
        return this;
    }

    /**
     * @see MapObjectMapperBuilder#build()
     */
    public MapObjectMapper build() {
        if (interceptor == null) {
            return new DefaultMapObjectMapper(packageName, buildPropertyModel(), buildTypeConverter());
        }
        return new InterceptingMapObjectMapper(packageName, buildPropertyModel(), buildTypeConverter(),
                interceptor);
    }

    /**
     * answers the property model. If no property model was setted, uses the
     * naming a setter styles
     */
    private PropertyModel buildPropertyModel() {
        if(propertyModel != null) {
            Ensure.that(setterStyle == null, "Either set the property model or the setter style, but not both");
            Ensure.that(namingStyle == null, "Either set the property model or the naming style, but not both");
            return propertyModel;
        }
        return new SimplePropertyModel(buildSetterStyle(), buildNamingStyle());
    }

    /**answers the naming style. Defaults to bean styles*/
    private NamingStyle buildNamingStyle() {
        return Nulls.coalesce(namingStyle, BeanNamingStyle.syle());
    }

    /**answers he setter style. Defaults to bean style*/
    private SetterStyle buildSetterStyle() {
        return Nulls.coalesce(setterStyle, BeanStyle.STYLE);
    }

    /**Wraps a list of converters into a composite converter*/
    private CompositeTypeConverter buildTypeConverter() {
        return new CompositeTypeConverter(converters);
    }
}
